@@ -14,6 +14,7 @@ if t.TYPE_CHECKING:
         ConcreteArtifactsManager,
     )
     from ansible.galaxy.collection.galaxy_api_proxy import MultiGalaxyAPIProxy
+    from ansible.galaxy.api import GalaxyAPI
 
 from ansible.galaxy.collection.gpg import get_signature_from_source
 from ansible.galaxy.dependency_resolution.dataclasses import (
@@ -25,10 +26,24 @@ from ansible.galaxy.dependency_resolution.versioning import (
     meets_requirements,
 )
 from ansible.module_utils.six import string_types
-from ansible.utils.version import SemanticVersion
+from ansible.utils.version import SemanticVersion, LooseVersion
 
 from collections.abc import Set
-from resolvelib import AbstractProvider
+
+try:
+    from resolvelib import AbstractProvider
+    from resolvelib import __version__ as resolvelib_version
+except ImportError:
+    class AbstractProvider:  # type: ignore[no-redef]
+        pass
+
+    resolvelib_version = '0.0.0'
+
+
+# TODO: add python requirements to ansible-test's ansible-core distribution info and remove the hardcoded lowerbound/upperbound fallback
+RESOLVELIB_LOWERBOUND = SemanticVersion("0.5.3")
+RESOLVELIB_UPPERBOUND = SemanticVersion("1.1.0")
+RESOLVELIB_VERSION = SemanticVersion.from_loose_version(LooseVersion(resolvelib_version))
 
 
 class PinnedCandidateRequests(Set):
@@ -59,11 +74,11 @@ class PinnedCandidateRequests(Set):
         return len(self._candidates)
 
 
-class CollectionDependencyProvider(AbstractProvider):
+class CollectionDependencyProviderBase(AbstractProvider):
     """Delegate providing a requirement interface for the resolver."""
 
     def __init__(
-            self,  # type: CollectionDependencyProvider
+            self,  # type: CollectionDependencyProviderBase
             apis,  # type: MultiGalaxyAPIProxy
             concrete_artifacts_manager=None,  # type: ConcreteArtifactsManager
             user_requirements=None,  # type: t.Iterable[Requirement]
@@ -155,7 +170,7 @@ class CollectionDependencyProvider(AbstractProvider):
         return False
 
     def identify(self, requirement_or_candidate):
-        # type: (Candidate | Requirement) -> str
+        # type: (t.Union[Candidate, Requirement]) -> str
         """Given requirement or candidate, return an identifier for it.
 
         This is used to identify a requirement or candidate, e.g.
@@ -166,12 +181,8 @@ class CollectionDependencyProvider(AbstractProvider):
         """
         return requirement_or_candidate.canonical_package_id
 
-    def get_preference(
-            self,  # type: CollectionDependencyProvider
-            resolution,  # type: Candidate | None
-            candidates,  # type: list[Candidate]
-            information,  # type: list[t.NamedTuple]
-    ):  # type: (...) -> float | int
+    def get_preference(self, *args, **kwargs):
+        # type: (t.Any, t.Any) -> t.Union[float, int]
         """Return sort key function return value for given requirement.
 
         This result should be based on preference that is defined as
@@ -179,6 +190,8 @@ class CollectionDependencyProvider(AbstractProvider):
         The lower the return value is, the more preferred this
         group of arguments is.
 
+        resolvelib >=0.5.3, <0.7.0
+
         :param resolution: Currently pinned candidate, or ``None``.
 
         :param candidates: A list of possible candidates.
@@ -194,6 +207,35 @@ class CollectionDependencyProvider(AbstractProvider):
             (dependend on) the requirement, or `None`
             to indicate a root requirement.
 
+        resolvelib >=0.7.0, < 0.8.0
+
+        :param identifier: The value returned by ``identify()``.
+
+        :param resolutions: Mapping of identifier, candidate pairs.
+
+        :param candidates: Possible candidates for the identifer.
+            Mapping of identifier, list of candidate pairs.
+
+        :param information: Requirement information of each package.
+            Mapping of identifier, list of named tuple pairs.
+            The named tuples have the entries ``requirement`` and ``parent``.
+
+        resolvelib >=0.8.0, <= 1.0.1
+
+        :param identifier: The value returned by ``identify()``.
+
+        :param resolutions: Mapping of identifier, candidate pairs.
+
+        :param candidates: Possible candidates for the identifer.
+            Mapping of identifier, list of candidate pairs.
+
+        :param information: Requirement information of each package.
+            Mapping of identifier, list of named tuple pairs.
+            The named tuples have the entries ``requirement`` and ``parent``.
+
+        :param backtrack_causes: Sequence of requirement information that were
+            the requirements that caused the resolver to most recently backtrack.
+
         The preference could depend on a various of issues, including
         (not necessarily in this order):
 
@@ -215,6 +257,10 @@ class CollectionDependencyProvider(AbstractProvider):
         the value is, the more preferred this requirement is (i.e. the
         sorting function is called with ``reverse=False``).
         """
+        raise NotImplementedError
+
+    def _get_preference(self, candidates):
+        # type: (list[Candidate]) -> t.Union[float, int]
         if any(
                 candidate in self._preferred_candidates
                 for candidate in candidates
@@ -224,8 +270,8 @@ class CollectionDependencyProvider(AbstractProvider):
             return float('-inf')
         return len(candidates)
 
-    def find_matches(self, requirements):
-        # type: (list[Requirement]) -> list[Candidate]
+    def find_matches(self, *args, **kwargs):
+        # type: (t.Any, t.Any) -> list[Candidate]
         r"""Find all possible candidates satisfying given requirements.
 
         This tries to get candidates based on the requirements' types.
@@ -237,15 +283,31 @@ class CollectionDependencyProvider(AbstractProvider):
         to find concrete candidates for this requirement. Of theres a
         pre-installed candidate, it's prepended in front of others.
 
+        resolvelib >=0.5.3, <0.6.0
+
         :param requirements: A collection of requirements which all of \
                              the returned candidates must match. \
                              All requirements are guaranteed to have \
                              the same identifier. \
                              The collection is never empty.
 
+        resolvelib >=0.6.0
+
+        :param identifier: The value returned by ``identify()``.
+
+        :param requirements: The requirements all returned candidates must satisfy.
+            Mapping of identifier, iterator of requirement pairs.
+
+        :param incompatibilities: Incompatible versions that must be excluded
+            from the returned list.
+
         :returns: An iterable that orders candidates by preference, \
                   e.g. the most preferred candidate comes first.
         """
+        raise NotImplementedError
+
+    def _find_matches(self, requirements):
+        # type: (list[Requirement]) -> list[Candidate]
         # FIXME: The first requirement may be a Git repo followed by
         # FIXME: its cloned tmp dir. Using only the first one creates
         # FIXME: loops that prevent any further dependency exploration.
@@ -255,8 +317,18 @@ class CollectionDependencyProvider(AbstractProvider):
         # The fqcn is guaranteed to be the same
         version_req = "A SemVer-compliant version or '*' is required. See https://semver.org to learn how to compose it correctly. "
         version_req += "This is an issue with the collection."
+
+        # If we're upgrading collections, we can't calculate preinstalled_candidates until the latest matches are found.
+        # Otherwise, we can potentially avoid a Galaxy API call by doing this first.
+        preinstalled_candidates = set()
+        if not self._upgrade and first_req.type == 'galaxy':
+            preinstalled_candidates = {
+                candidate for candidate in self._preferred_candidates
+                if candidate.fqcn == fqcn and
+                all(self.is_satisfied_by(requirement, candidate) for requirement in requirements)
+            }
         try:
-            coll_versions = self._api_proxy.get_collection_versions(first_req)
+            coll_versions = [] if preinstalled_candidates else self._api_proxy.get_collection_versions(first_req)  # type: t.Iterable[t.Tuple[str, GalaxyAPI]]
         except TypeError as exc:
             if first_req.is_concrete_artifact:
                 # Non hashable versions will cause a TypeError
@@ -334,19 +406,20 @@ class CollectionDependencyProvider(AbstractProvider):
             reverse=True,  # prefer newer versions over older ones
         )
 
-        preinstalled_candidates = {
-            candidate for candidate in self._preferred_candidates
-            if candidate.fqcn == fqcn and
-            (
-                # check if an upgrade is necessary
-                all(self.is_satisfied_by(requirement, candidate) for requirement in requirements) and
+        if not preinstalled_candidates:
+            preinstalled_candidates = {
+                candidate for candidate in self._preferred_candidates
+                if candidate.fqcn == fqcn and
                 (
-                    not self._upgrade or
-                    # check if an upgrade is preferred
-                    all(SemanticVersion(latest.ver) <= SemanticVersion(candidate.ver) for latest in latest_matches)
+                    # check if an upgrade is necessary
+                    all(self.is_satisfied_by(requirement, candidate) for requirement in requirements) and
+                    (
+                        not self._upgrade or
+                        # check if an upgrade is preferred
+                        all(SemanticVersion(latest.ver) <= SemanticVersion(candidate.ver) for latest in latest_matches)
+                    )
                 )
-            )
-        }
+            }
 
         return list(preinstalled_candidates) + latest_matches
 
@@ -424,3 +497,52 @@ class CollectionDependencyProvider(AbstractProvider):
             self._make_req_from_dict({'name': dep_name, 'version': dep_req})
             for dep_name, dep_req in req_map.items()
         ]
+
+
+# Classes to handle resolvelib API changes between minor versions for 0.X
+class CollectionDependencyProvider050(CollectionDependencyProviderBase):
+    def find_matches(self, requirements):  # type: ignore[override]
+        # type: (list[Requirement]) -> list[Candidate]
+        return self._find_matches(requirements)
+
+    def get_preference(self, resolution, candidates, information):  # type: ignore[override]
+        # type: (t.Optional[Candidate], list[Candidate], list[t.NamedTuple]) -> t.Union[float, int]
+        return self._get_preference(candidates)
+
+
+class CollectionDependencyProvider060(CollectionDependencyProviderBase):
+    def find_matches(self, identifier, requirements, incompatibilities):  # type: ignore[override]
+        # type: (str, t.Mapping[str, t.Iterator[Requirement]], t.Mapping[str, t.Iterator[Requirement]]) -> list[Candidate]
+        return [
+            match for match in self._find_matches(list(requirements[identifier]))
+            if not any(match.ver == incompat.ver for incompat in incompatibilities[identifier])
+        ]
+
+    def get_preference(self, resolution, candidates, information):  # type: ignore[override]
+        # type: (t.Optional[Candidate], list[Candidate], list[t.NamedTuple]) -> t.Union[float, int]
+        return self._get_preference(candidates)
+
+
+class CollectionDependencyProvider070(CollectionDependencyProvider060):
+    def get_preference(self, identifier, resolutions, candidates, information):  # type: ignore[override]
+        # type: (str, t.Mapping[str, Candidate], t.Mapping[str, t.Iterator[Candidate]], t.Iterator[t.NamedTuple]) -> t.Union[float, int]
+        return self._get_preference(list(candidates[identifier]))
+
+
+class CollectionDependencyProvider080(CollectionDependencyProvider060):
+    def get_preference(self, identifier, resolutions, candidates, information, backtrack_causes):  # type: ignore[override]
+        # type: (str, t.Mapping[str, Candidate], t.Mapping[str, t.Iterator[Candidate]], t.Iterator[t.NamedTuple], t.Sequence) -> t.Union[float, int]
+        return self._get_preference(list(candidates[identifier]))
+
+
+def _get_provider():  # type () -> CollectionDependencyProviderBase
+    if RESOLVELIB_VERSION >= SemanticVersion("0.8.0"):
+        return CollectionDependencyProvider080
+    if RESOLVELIB_VERSION >= SemanticVersion("0.7.0"):
+        return CollectionDependencyProvider070
+    if RESOLVELIB_VERSION >= SemanticVersion("0.6.0"):
+        return CollectionDependencyProvider060
+    return CollectionDependencyProvider050
+
+
+CollectionDependencyProvider = _get_provider()
